<?php

/* phpBlip Class 0.1 based on phpYoutube Class 1.1 based on phpFlickr Class 1.6.1
 * Modified by Shawn Van Every (vanevery@walking-productions.com)
 * phpFlickr written by Dan Coulter (dan@dancoulter.com)
 * phpYoutube written by Erik de Vries (info@erikdevries.nl)
 * Released under GNU Lesser General Public License (http://www.gnu.org/copyleft/lgpl.html)
 * Uses Blip.TV REST API http://wiki.blip.tv/index.php/Blip.tv_API_2.0
 */

if (session_id() == "") {
    session_start();
}
require_once('xml.php');

// Decides which include path delimiter to use.  Windows should be using a semi-colon
// and everything else should be using a colon.  If this isn't working on your system,
// comment out this if statement and manually set the correct value into $path_delimiter.
if (strpos(__FILE__, ':') !== false) {
    $path_delimiter = ';';
} else {
    $path_delimiter = ':';
}

// This will add the packaged PEAR files into the include path for PHP, allowing you
// to use them transparently.  This will prefer officially installed PEAR files if you
// have them.  If you want to prefer the packaged files (there shouldn't be any reason
// to), swap the two elements around the $path_delimiter variable.  If you don't have
// the PEAR packages installed, you can leave this like it is and move on.

ini_set('include_path', ini_get('include_path') . $path_delimiter . dirname(__FILE__) . '/PEAR');

// If you have problems including the default PEAR install (like if your open_basedir
// setting doesn't allow you to include files outside of your web root), comment out
// the line above and uncomment the next line:

// ini_set('include_path', dirname(__FILE__) . '/PEAR' . $path_delimiter . ini_get('include_path'));

class phpBlip 
{
    var $REST = 'http://www.blip.tv/?section=posts&cmd=view&skin=api';
    var $xml_parser;
    var $req;
    var $response;
    var $parsed_response;
    var $cache = false;
    var $cache_db = null;
    var $cache_table = null;
    var $cache_dir = null;
    var $cache_expire = null;
    var $error_code;
    var $error_msg;
    var $php_version;
    
    function phpBlip() 
    {        
        //Find the PHP version and store it for future reference
        $this->php_version = explode("-", phpversion());
        $this->php_version = explode(".", $this->php_version[0]);
        
        //All calls to the API are done via the POST method using the PEAR::HTTP_Request package.
        require_once 'HTTP/Request.php';
        $this->req =& new HTTP_Request();
                
        //setup XML parser using Aaron Colflesh's XML class.
        //$this->xml_parser = new xml(false, true, true);
        //$this->xml_parser = new xml();
    }
    
    function enableCache($type, $connection, $cache_expire = 600, $table = 'blip_cache') 
    {
        // Turns on caching.  $type must be either "db" (for database caching) or "fs" (for filesystem).
        // When using db, $connection must be a PEAR::DB connection string. Example:
        //      "mysql://user:password@server/database"
        // If the $table, doesn't exist, it will attempt to create it.
        // When using file system, caching, the $connection is the folder that the web server has write
        // access to. Use absolute paths for best results.  Relative paths may have unexpected behavior 
        // when you include this.  They'll usually work, you'll just want to test them.
        if ($type == 'db') {
            require_once 'DB.php';
            $db =& DB::connect($connection);
            if (PEAR::isError($db)) {
                die($db->getMessage());
            }
            
            $db->query("
                CREATE TABLE IF NOT EXISTS `$table` (
                    `request` CHAR( 35 ) NOT NULL ,
                    `response` MEDIUMTEXT NOT NULL ,
                    `expiration` DATETIME NOT NULL ,
                    INDEX ( `request` )
                ) TYPE = MYISAM");
            $db->query("DELETE FROM $table WHERE expiration < DATE_SUB(NOW(), INTERVAL $cache_expire second)");
            if (strpos($connection, 'mysql') !== false) {
                $db->query('OPTIMIZE TABLE ' . $table);
            }
            $this->cache = 'db';
            $this->cache_db = $db;
            $this->cache_table = $table;
        } elseif ($type = 'fs') {
            $this->cache = 'fs';
            $connection = realpath($connection);
            $this->cache_dir = $connection;
            if ($dir = opendir($this->cache_dir)) {
                while ($file = readdir($dir)) {
                    if (substr($file, -6) == '.cache' && ((filemtime($this->cache_dir . '/' . $file) + $cache_expire) < time()) ) {
                        unlink($this->cache_dir . '/' . $file);
                    }
                }
            }
        }
        $this->cache_expire = $cache_expire;
    }
    
    function getCached($request) 
    {
        //Checks the database or filesystem for a cached result to the request.
        //If there is no cache result, it returns a value of false. If it finds one,
        //it returns the unparsed XML.
        $reqhash = md5(serialize($request));
        if ($this->cache == 'db') {
            $result = $this->cache_db->getOne("SELECT response FROM " . $this->cache_table . " WHERE request = '" . $reqhash . "'");
            if (!empty($result)) {
                return $result;
            }
        } elseif ($this->cache == 'fs') {
            $file = $this->cache_dir . '/' . $reqhash . '.cache';
            if (file_exists($file)) {
				if ($this->php_version[0] > 4 || ($this->php_version[0] == 4 && $this->php_version[1] >= 3)) {
					return file_get_contents($file);
				} else {
					return implode('', file($file));
				}
            }
        }
        return false;
    }
    
    function cache($request, $response) 
    {
        //Caches the unparsed XML of a request.
        $reqhash = md5(serialize($request));
        if ($this->cache == 'db') {
            $this->cache_db->query("DELETE FROM $this->cache_table WHERE request = '$reqhash'");
            $sql = "INSERT INTO " . $this->cache_table . " (request, response, expiration) VALUES ('$reqhash', '" . str_replace("'", "''", $response) . "', '" . strftime("%Y-%m-%d %H:%M:%S") . "')";
            $this->cache_db->query($sql);
        } elseif ($this->cache == "fs") {
            $file = $this->cache_dir . "/" . $reqhash . ".cache";
            $fstream = fopen($file, "w");
            $result = fwrite($fstream,$response);
            fclose($fstream);
            return $result;
        }
        return false;
    }
    
    
    function request($args = array(), $nocache = false) 
    {
        //Sends a request to Blip's REST endpoint via GET.
        $request_url = $this->REST;
        
        if (!($this->response = $this->getCached($args)) || $nocache) {
            foreach ($args as $key => $data) 
            {
                if (!empty($data))
                {
	                $request_url .= "&" . $key . "=" . $data;
	            }
            }
	     	
	     	//echo $request_url;
	     	$this->req->setURL($request_url);            
         
         	//Send Request
            if ($this->req->sendRequest()) {
                $this->response = $this->req->getResponseBody();
                $this->cache($args, $this->response);
            } else {
                die("There has been a problem sending your command to the server.");
            }
        }
        return $this->response;
    }
    
    function parse_response($xml = NULL) 
    {
        //Sends response data through XML parser and returns an associative array.
        if ($xml === NULL) {
            $xml = $this->response;
        }
		$this->parsed_response = $this->parse($xml);

        //Check for an error and die if it finds one.
        /*
        if (!empty($this->parsed_response['ut_response']['error']) && $this->die_on_error) {
            die("The Blip.tv API returned error code #" . $this->parsed_response['ut_response']['error']['code'] . ": " . $this->parsed_response['ut_response']['error']['description']);
        } elseif (!empty($this->parsed_response['ut_response']['error'])) {
			$this->error_code = $this->parsed_response['ut_response']['error']['code'];
			$this->error_msg = "The Blip.tv API returned error code #" . $this->parsed_response['ut_response']['error']['code'] . ": " . $this->parsed_response['ut_response']['error']['description'];
			return false;
        } else {
			$this->error_code = false;
			$this->error_msg = false;
        }
        */
                
        return $this->parsed_response;
    }
        
    function setProxy($server, $port) 
    {
        // Sets the proxy for all phpBlip calls.
        $this->req->setProxy($server, $port);
    }
        
    function getErrorCode() 
    {
		// Returns the error code of the last call.  If the last call did not
		// return an error. This will return a false boolean.
		return $this->error_code;
    }
    
    function getErrorMsg() 
    {
		// Returns the error message of the last call.  If the last call did not
		// return an error. This will return a false boolean.
		return $this->error_msg;
    }
    
    function xml_get_assets($xml)
    {
    	// Return Array
    	$assets = array();
    	$asset_number = -1;
    	$found_assets = false;
    	
    	// Get to Payload
    	//print_r($xml);
    	for ($i = 0; $i < sizeof($xml) && !$found_assets; $i++)
    	{
    		if ($xml[$i]->tag == "response")
    		{
    			if (!empty($xml[$i]->next))
    			{
	    			$response = $xml[$i]->next;
	    			for ($r = 0; $r < sizeof($response) && !$found_assets; $r++)
	    			{
						if ($response[$r]->tag == "payload")
						{
							if (!empty($response[$r]->next))
							{
								$xml_assets = $response[$r]->next;
								$found_assets = true;
								
								for ($a = 0; $a < sizeof($xml_assets); $a++)
								{
									if ($xml_assets[$a]->tag == "asset")
									{
										if (!empty($xml_assets[$a]->next))
										{								
											$asset_number++;
											$xml_asset_tags = $xml_assets[$a]->next;
											for ($t = 0; $t < sizeof($xml_asset_tags); $t++)
											{
												//echo($asset_number . ":" . $xml_asset_tags[$t]->tag . ":" . $xml_asset_tags[$t]->value . "\n");
												$assets[$asset_number][$xml_asset_tags[$t]->tag] = $xml_asset_tags[$t]->value;
											}
										}
									}	    			
								}
							}
						}    
		    		}
    			}
    		}
    	}
    	return $assets;
    }
    
    function xml_get_post_data($xml)
    {
    	// Return Array
    	$post_data = array();
    	$found_asset = false;
    	
    	// Get to Payload
    	//print_r($xml);
    	for ($i = 0; $i < sizeof($xml) && !$found_asset; $i++)
    	{
    		if ($xml[$i]->tag == "response")
    		{
    			if (!empty($xml[$i]->next))
    			{
	    			$response = $xml[$i]->next;
	    			for ($r = 0; $r < sizeof($response) && !$found_asset; $r++)
	    			{
						if ($response[$r]->tag == "payload")
						{
							if (!empty($response[$r]->next))
							{
								$xml_assets = $response[$r]->next;
								$found_asset = true;
								
								for ($a = 0; $a < sizeof($xml_assets); $a++)
								{
									if ($xml_assets[$a]->tag == "asset")
									{
										if (!empty($xml_assets[$a]->next))
										{								
											$xml_asset_tags = $xml_assets[$a]->next;
											for ($t = 0; $t < sizeof($xml_asset_tags); $t++)
											{
												if ($xml_asset_tags[$t]->tag == "id")
												{
													$post_data["id"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "item_id")
												{
													$post_data["item_id"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "guid")
												{
													$post_data["guid"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "item_type")
												{
													$post_data["item_type"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "deleted")
												{
													$post_data["deleted"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "views")
												{
													$post_data["views"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "title")
												{
													$post_data["title"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "modified")
												{
													$post_data["modified"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "timestamp")
												{
													$post_data["timestamp"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "latitude")
												{
													$post_data["latitude"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "longitude")
												{
													$post_data["longitude"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "explicit")
												{
													$post_data["explicit"] = $xml_asset_tags[$t]->value;
												}
												else if ($xml_asset_tags[$t]->tag == "views")
												{
													$post_data["description"] = $xml_asset_tags[$t]->value;
												}
												/*
												links
												license
												createdBy
												tags
												keyFrames
												*/
												else if ($xml_asset_tags[$t]->tag == "mediaList")
												{
													$mediaList = $xml_asset_tags[$t]->next;
													//print_r($mediaList);
													$media = array();
													
													
													for ($m = 0; $m < sizeof($mediaList); $m++)
													{
														$mediatags = $mediaList[$m]->next;
														for ($mt = 0; $mt < sizeof($mediatags); $mt++)
														{
															if ($mediatags[$mt]->tag == "link")
															{
																$link_attribs = $mediatags[$mt]->attributes;
																$media[$m]["link_href"] = $link_attribs["href"];
																$media[$m]["link_type"] = $link_attribs["type"];
															}
															else
															{
																$media[$m][$mediatags[$mt]->tag] = $mediatags[$mt]->value;
															}
														}
													}
													$post_data["media"] = $media;
												}												
											}
										}
									}	    			
								}
							}
						}    
		    		}
    			}
    		}
    	}
    	return $post_data;    
    }
    
	function xml2array($xml_string)
	{
	   $parser = xml_parser_create();
	   xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, 0);
	   xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE, 1);
	   xml_parse_into_struct($parser, $xml_string, $xml_values);
	   xml_parser_free($parser);
	   $xmlclass = array();
	   $lastobj = array();
	   $nowobj = &$xmlclass;
	
	   foreach($xml_values as $Xml_Key => $xml_value)
	   {
		   $Index = count($nowobj);
		   if($xml_value["type"] == "complete")
		   {
			   $nowobj[$Index] = new xml;
			   if (!empty($xml_value["tag"]))
				   $nowobj[$Index]->tag = $xml_value["tag"];
			   if (!empty($xml_value["value"]))		   
				   $nowobj[$Index]->value = $xml_value["value"];
			   if (!empty($xml_value["attributes"]))
				   $nowobj[$Index]->attributes = $xml_value["attributes"];
		   }
		   elseif($xml_value["type"] == "open")
		   {
			   $nowobj[$Index] = new xml;
			   if (!empty($xml_value["tag"]))
				   $nowobj[$Index]->tag = $xml_value["tag"];
			   if (!empty($xml_value["value"]))		   
				   $nowobj[$Index]->value = $xml_value["value"];
			   if (!empty($xml_value["attributes"]))
				   $nowobj[$Index]->attributes = $xml_value["attributes"];
			   $nowobj[$Index]->next = array();
			   $lastobj[count($lastobj)] = &$nowobj;
			   $nowobj = &$nowobj[$Index]->next;
		   }
		   elseif($xml_value["type"] == "close")
		   {
			   $nowobj = &$lastobj[count($lastobj) - 1];
			   unset($lastobj[count($lastobj) - 1]);
		   }
		   
	   }
	   return $xmlclass;
	}
	
	function parse($xml_string)
	{
		return $this->xml2array($xml_string);
	}
    
    
    /* 
        These functions are implementations of blip calls.
        For method documentation, including arguments, visit 
        http://wiki.blip.tv/index.php/Blip.tv_API_2.0
    */
     
     // http://wiki.blip.tv/index.php/Find_posts
     // http://www.blip.tv/?section=posts&cmd=view&skin=api
    function find_posts($userid = null, $search = null, $explicit = false, $language_code = "en", $category = null, $license = null, $has_media = 1)
    {
    	$this->request(array("userid" => $userid, "search" => $search, "explicit" => $explicit, "language_code" => $language_code, "category" => $category, "license" => $license, "has_media" => $has_media));

        $result = $this->parse_response();        
      	$return = $this->xml_get_assets($result);
        return $return;
    }
    
    // http://wiki.blip.tv/index.php/Find_post
    // http://www.blip.tv/?section=posts&cmd=view&id=...
	function find_post($id)
	{
    	$this->request(array("id" => $id));
        $result = $this->parse_response();        
      	$return = $this->xml_get_post_data($result);
        return $return;	
	}

}
?>